pub mod session;
use session::*;

use std::{
  collections::BTreeMap,
  env::var,
  fs::File,
  io::{BufRead, BufReader, Result, Write},
  path::PathBuf,
};
use tmux_interface::{commands::tmux::StdIO, KillSession, ListClients, Tmux};

fn separate(line: Result<String>) -> Option<(String, String)> {
  let line_ = line.unwrap();
  let (s1, s2) = line_.split_once(" ")?;
  Some((s1.to_owned(), s2.to_owned()))
}

fn set_projects(path: &PathBuf, projects: &BTreeMap<String, String>) {
  let mut project_lines = projects
    .iter()
    .map(|(project_name, project_path)| {
      let mut project_line: String = project_name.to_owned();
      project_line.push_str(" ");
      project_line.push_str(project_path);
      return project_line;
    })
    .collect::<Vec<_>>()
    .join("\n");

  project_lines.push('\n');

  let mut file = File::create(path).expect("Projects list could not be created.");
  file
    .write(project_lines.as_bytes())
    .expect("Projects list could not be written to.");
}

fn get_projects(path: &PathBuf) -> BTreeMap<String, String> {
  let reader = BufReader::new(match File::open(path) {
    Ok(file) => file,
    Err(_) => {
      File::create(path).expect("Projects list could not be created.");
      File::open(path).expect("Projects list could not be opened.")
    }
  });

  reader.lines().filter_map(separate).collect()
}

#[derive(Debug)]
enum Mode {
  Find,
  Add,
  Delete,
  Base,
}

fn main() {
  let args: Vec<String> = std::env::args().collect();

  let mode: Mode = match args.get(1) {
    Some(flag) => match flag.as_str() {
      "-a" => Mode::Add,
      "-d" => Mode::Delete,
      "-b" => Mode::Base,
      _ => Mode::Find,
    },
    _ => Mode::Find,
  };

  let project_file_path = PathBuf::from(match var("PROJECT_MANAGER_PATH") {
    Ok(path) => path,
    _ => match var("XDG_DATA_HOME") {
      Ok(data_home) => [data_home, "projects".to_string()].join("/"),
      _ => match var("HOME") {
        Ok(home) => [home, "projects".to_string()].join("/"),
        _ => "/tmp/projects".to_string(),
      },
    },
  });
  let mut projects = get_projects(&project_file_path);

  let old_session_name = String::from_utf8(
    Tmux::with_command(
      ListClients::new()
        .target_session(std::env::var("TMUX_PANE").unwrap_or("".to_string()))
        .format("#S"),
    )
    .output()
    .unwrap()
    .stdout(),
  )
  .unwrap()
  .strip_suffix("\n")
  .unwrap_or("base")
  .to_string();

  let old_session_path = String::from_utf8(
    Tmux::with_command(
      ListClients::new()
        .target_session(std::env::var("TMUX_PANE").unwrap_or("".to_string()))
        .format("#{pane_start_path}"),
    )
    .output()
    .unwrap()
    .stdout(),
  )
  .unwrap()
  .strip_suffix("\n")
  .unwrap_or("~")
  .to_string();

  let project_name = match mode {
    Mode::Add => add_project(&mut projects, args.get(2)),
    Mode::Delete => {
      delete_project(&mut projects, &old_session_path);
      None
    }
    Mode::Base => None,
    Mode::Find => find_projects(&projects, args.get(1), &old_session_path),
  };

  focus_project(&projects, project_name);

  set_projects(&project_file_path, &projects);

  match mode {
    Mode::Delete => Tmux::with_command(KillSession::new().target_session(old_session_name))
      .stderr(Some(StdIO::Null))
      .status()
      .unwrap()
      .success(),
    _ => true,
  };

  match true {
    true => Tmux::with_command(KillSession::new().target_session("PROJECTS"))
      .stderr(Some(StdIO::Null))
      .status()
      .unwrap()
      .success(),
    false => false,
  };
}
